/* -----------------------------------------

   _  _ ____ _    _    
   |__| |___ |    |    
   |  | |___ |___ |___ 
   
   HELL: High-Energy Large Logarithms

   Author: Marco Bonvini

   Computes Delta P_res = P_res - P_FixedOrder
   from a file containing parameters

   ----------------------------------------- */

#pragma once

#include <iostream>
#include <map>
#include "math/matrix.hh"
#include "hell_tables.hh"


using namespace std;


#define gg entry11
#define gq entry12
#define qg entry21
#define qq entry22



namespace HELL {

  const double CA = 3.;
  const double CF = 0.5*CA - 0.5/CA;
  enum Order {
    none = -1,
    LO = 0,
    NLO = 1,
    NNLO = 2
  };
  enum LogOrder {
    LL = 0,
    NLL = 1
  };






  // Class for fixed nf.
  //
  class HELLnf {
  private:
    int _nf;
    int _order;
    double _damping;
    bool _momcons;
    vector<double> _alphas, _alphas2;
    string datapath_;
    map<int,HellTableP*> HTP;
    map<string,map<int,HellTableP*> > HTP2;
    map<int,HellTableP*>::iterator itHTP;
    int alphas_interpolation(double as, vector<double> vas, double &factor);
    void ReadTable (int k);
    void ReadTable2(int k, string id);
    double xdeltaPplus(double as, double x);
    double xdeltaPqg  (double as, double x);
    double xdeltaC(double as, double x, string id);
    sqmatrix<double>   DeltaPLL     (double x,  double as, Order matched_to_fixed_order=LO);
    sqmatrix<double>   DeltaPNLL    (double x,  double as, Order matched_to_fixed_order=NLO);
    map<double,double> MCpl, MCqg;
    map<double,double>::iterator itMC;
    friend double gplintegrand(double,void*);
    friend double gqgintegrand(double,void*);
    double gpl2(double as);
    double gqg2(double as);
  public:
    HELLnf(int nf, LogOrder order, string datapath="./data") : _nf(nf), _order(order), _damping(0), _momcons(false)
    { Init(datapath); }
    ~HELLnf();
    //
    void Init(string datapath);
    //
    int GetOrder() {return _order;};
    int GetNf() {return _nf;};
    void GetAvailableAlphas(vector<double> &as1, vector<double> &as2);
    void SetLargexDamping(double n=2) { _damping=n; } // multiplies deltaP by (1-x)^n
    void SetMomentumConservation(bool enable=true) { _momcons=enable; }
    //
    // Delta Splitting functions
    double deltaPplus(double as, double x);
    double xdeltaPpUqg(double as, double x); // remove this
    sqmatrix<double> DeltaP(double as, double x);//, Order matched_to_fixed_order = NLO);
    //
    // Delta Coefficient functions (the quark coefficients are found multiplying by CF/CA)
    // these are delta2, to be matched to NNLO
    double deltaC2g  (double as, double  x);  // DIS F2
    double deltaCLg  (double as, double  x);  // DIS FL
    double deltaC2gold  (double as, double  x);  // DIS F2
    double deltaCLgold  (double as, double  x);  // DIS FL
    //
  };



};

