#include <iomanip>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <cmath>
#include <cstdlib>

#include "Pqg.hh"
#include "math/borel.hh"
#include "gammaNLO.hh"
#include "h_coefficients.hh"


using namespace std;



namespace HELL {


  const double CA = 3.;
  const double CF = 4./3.;


  dcomplex hRC(dcomplex gamma, dcomplex gammadot, hCoeff hC, int nPade=8) {
    int Ncoeff = hC.hk().size();
    dcomplex *coeff_ck   = new dcomplex[Ncoeff];
    dcomplex *sqbr_gamma = new dcomplex[Ncoeff];
    dcomplex gammadot_over_gamma2 = gammadot/gamma/gamma;
    sqbr_gamma[0] = 1.;
    coeff_ck[0] = hC.hk(0);
    for(int k=1; k<Ncoeff; k++) {
      sqbr_gamma[k] = gamma * ( 1. + (k-1.)*gammadot_over_gamma2 ) * sqbr_gamma[k-1];
      coeff_ck[k] = hC.hk(k) * sqbr_gamma[k];
      //cout << coeff_ck[k] << endl;
    }
    //double Lambda = 20.;
    //nPade = Ncoeff/2-1;
    //nPade = 8;         // !!!!!! <<=== parameter here!!!
    //nPade = 7;         // !!!!!! <<=== parameter here!!!
    SetBorelOrder(2);  // !!!!!! <<=== parameter here: works with 1 or 2 
    //dcomplex borelsum = BorelPadeSum(1., nPade, coeff_ck, Ncoeff);
    dcomplex borelsum = BorelPadeSum(1., nPade, nPade-1, coeff_ck, Ncoeff);
    //dcomplex borelsum = BorelPadeSum(1., nPade, 1, coeff_ck, Ncoeff);
    //dcomplex borelsum = BorelTruncatedSumCUBA (1., hqgNcoeff-1, Lambda, coeff_ck, Ncoeff);
    //dcomplex borelsum = BorelTruncatedSumGauss(1., hqgNcoeff-1, Lambda, coeff_ck, Ncoeff);
    //dcomplex borelsum = BorelTruncatedSumCUBA (1., 20, Lambda, coeff_ck, Ncoeff);
    //dcomplex borelsum = BorelTruncatedSumGauss(1., 13, Lambda, coeff_ck, Ncoeff); // Peraro like
    return borelsum;
  }


  dcomplex fmom(dcomplex N) {
    return 1./N;
    //double k = 1;
    //return pow(4.*N/pow(N+1.,2),k);
  }

  dcomplex gamma0(dcomplex N, double nf) {
    return gamma0pl(N+1., 0) + fmom(N+1.)*nf*(2*CF/CA-1)/6./M_PI;
  }
  dcomplex gamma0singNoDamp(dcomplex N, double nf) {
    double momcons = 0;
    double c = (11*CA+2*nf*(1-2*CF/CA))/12.;
    dcomplex fixedorderpole = ( CA/N - c*fmom(N+1.) - momcons*fmom(N)*(CA-c*fmom(2.)) )/M_PI;
    return fixedorderpole;
  }
  dcomplex gamma0sing(dcomplex N, double nf) {
    return gamma0singNoDamp(N,nf);
    return gamma0singNoDamp(N,nf)/(N+1.);
    return gamma0singNoDamp(N,nf) - 2.*gamma0singNoDamp(N+1.,nf) + gamma0singNoDamp(N+2.,nf);
  }
  dcomplex gamma1sing(dcomplex N, double nf) {
    double c1 = nf*(26.*CF-23.*CA)/36.;
    double c2 = 0;
    dcomplex fixedorderpole = ( c1/N - c2*fmom(N+1.) )/M_PI/M_PI;
    return fixedorderpole;
  }

  double beta0(int nf) {
    return (11.*CA - 2.*nf)/12./M_PI;
  }
  dcomplex DeltaFRes(dcomplex Delta_gammaLO_BatNLO, dcomplex Delta_gammaLOdot_BatNLO, dcomplex N, double as, double nf, process proc, int ordersub=0, int nPade=8) {
    hCoeff hC(proc);
    double h0 = hC.hk(0);
    double h1 = hC.hk(1);
    //
    double b0 = beta0(nf);
    dcomplex fixedorderpole = gamma0sing(N,nf);
    //fixedorderpole = gamma0(N,nf) - gamma0(N+1.,nf);
    //fixedorderpole = gamma0(N,nf) - 2.*gamma0(N+1.,nf) + gamma0(N+2.,nf);
    //fixedorderpole = gamma0(N,nf) - 3.*gamma0(N+1.,nf) + 3.*gamma0(N+2.,nf) - gamma0(N+3.,nf);
    //*
    // Marco version
    dcomplex gammaLO_BatNLO    = Delta_gammaLO_BatNLO    + as*fixedorderpole;//  + as*as*gamma1sing(N,nf);
    dcomplex gammaLOdot_BatNLO = Delta_gammaLOdot_BatNLO +    fixedorderpole;
    //
    //gammaLO_BatNLO /= N+1.; // hack! large-N damping
    //gammaLOdot_BatNLO /= N+1.; // hack! large-N damping
    //gammaLOdot_BatNLO = gammaLO_BatNLO/as; //hack!!!
    //
    gammaLOdot_BatNLO *= -b0*as*as;  // convert deriv wrt as to deriv wrt t
    //gammaLOdot_BatNLO = 0;
    dcomplex res = hRC(gammaLO_BatNLO, gammaLOdot_BatNLO, hC, nPade);
    /*/
    // Richard version
    dcomplex FOgammaLO  = as*gamma0pl(N+1.,0);
    dcomplex gammaLO_BatNLO    = Delta_gammaLO_BatNLO    + FOgammaLO;
    dcomplex gammaLOdot_BatNLO = Delta_gammaLOdot_BatNLO + FOgammaLO/as;
    gammaLOdot_BatNLO *= -b0*as*as;  // convert deriv wrt as to deriv wrt t
    FOgammaLO -= as*gamma0sing(N,0);
    dcomplex FOgammaLOdot  = -b0*as*FOgammaLO;
    //gammaLOdot_BatNLO = 0;
    //FOgammaLOdot = 0;
    dcomplex res = hRC(gammaLO_BatNLO, gammaLOdot_BatNLO, hC) - hRC(FOgammaLO, FOgammaLOdot, hC) + h0;
    //*/
    //
    //cout << res << endl;
    //if(res != 1.) exit(0);
    if(ordersub>0) res -= h0;
    if(ordersub>1) res -= h1*as*fixedorderpole;
    if(ordersub>2) res -= 0;
    return res;
  }


  dcomplex DeltaGammaResNLOqg(dcomplex Delta_gammaLO_BatNLO, dcomplex Delta_gammaLOdot_BatNLO, dcomplex N, double as, double nf, dcomplex Delta_gammaNLO) {
    dcomplex res = DeltaFRes(Delta_gammaLO_BatNLO, Delta_gammaLOdot_BatNLO, N, as, nf, gamma_qg, 2, 8);
    return res * as*nf/3./M_PI;
  }
  dcomplex DeltaUqg(dcomplex Delta_gammaLO_BatNLO, dcomplex Delta_gammaLOdot_BatNLO, dcomplex N, double as, double nf, dcomplex Delta_gammaNLO) {
    dcomplex res = DeltaFRes(Delta_gammaLO_BatNLO, Delta_gammaLOdot_BatNLO, N, as, nf, Uqg, 1, 7); // should be 2, but then it's doubly counted in C2, so it's 1
    //dcomplex res = DeltaFRes(Delta_gammaLO_BatNLO, Delta_gammaLOdot_BatNLO, N, as, nf, Uqg, 0, 7);
    return res * as*nf/3./M_PI;
  }
  dcomplex DeltaGammaPlusUqg(dcomplex Delta_gammaLO_BatNLO, dcomplex Delta_gammaLOdot_BatNLO, dcomplex N, double as, double nf, dcomplex Delta_gammaNLO) {
    double b0 = beta0(nf);
    double c = (11*CA+2*nf*(1-2*CF/CA))/12.;
    dcomplex fixedorderpole = ( CA/N - c*fmom(N+1.) )/M_PI;
    dcomplex gammaLO_BatNLO    = Delta_gammaLO_BatNLO    + as*fixedorderpole;
    dcomplex gammaLOdot_BatNLO = Delta_gammaLOdot_BatNLO +    fixedorderpole;
    //gammaLOdot_BatNLO = gammaLO_BatNLO/as; //hack!!!
    gammaLOdot_BatNLO *= -b0*as*as;  // convert deriv wrt as to deriv wrt t
    dcomplex r = -gammaLOdot_BatNLO / gammaLO_BatNLO;
    dcomplex gamma0 = fixedorderpole;
    dcomplex gamma1 = nf*(26.*CF-23.*CA)/36./M_PI/M_PI/N;
    dcomplex gammaNLO = as*gamma0 + as*as*gamma1 + Delta_gammaNLO;
    hCoeff hC(Uqg);
    double h0 = hC.hk(0);
    double h1 = hC.hk(1);
    //dcomplex extraU  =  h0*as/(r + gammaLO_BatNLO);
    dcomplex extraU  =  h0*as/(r + gammaNLO);
    dcomplex extraU0 =  h0/(b0+gamma0);
    dcomplex extraU1 = -h0*gamma1/gamma0/(b0+gamma0);
    extraU = extraU0 = extraU1 = 0; // hack!
    dcomplex U  = nf/3./M_PI * ( as*DeltaFRes(Delta_gammaLO_BatNLO, Delta_gammaLOdot_BatNLO, N, as, nf, Uqg, 0, 7) + extraU );
    dcomplex U0 = nf/3./M_PI * ( extraU0 );
    dcomplex U1 = nf/3./M_PI * ( extraU1 + h1 );
    return gammaNLO*U - as*gamma0*U0 - as*as*(gamma1*U0 + gamma0*U1);
    return (gammaNLO+r)*U - as*(gamma0+b0)*U0 - as*as*(gamma1*(1.+b0/gamma0)*U0 + (gamma0+b0)*U1);
  }
  dcomplex oldDeltaCLRes(dcomplex Delta_gammaLO_BatNLO, dcomplex Delta_gammaLOdot_BatNLO, dcomplex N, double as, double nf, dcomplex Delta_gammaNLO) {
    dcomplex res = DeltaFRes(Delta_gammaLO_BatNLO, Delta_gammaLOdot_BatNLO, N, as, nf, CL, 2, 8);
    return res * as*nf/3./M_PI;
  }
  dcomplex oldDeltaC2Res(dcomplex Delta_gammaLO_BatNLO, dcomplex Delta_gammaLOdot_BatNLO, dcomplex N, double as, double nf, dcomplex Delta_gammaNLO) {
    dcomplex res = DeltaFRes(Delta_gammaLO_BatNLO, Delta_gammaLOdot_BatNLO, N, as, nf, C2, 2, 8);
    return res * as*nf/3./M_PI;
  }



};

